@@ -43,7 +43,6 @@ extern "C" void whiskermenu_construct(XfcePanelPlugin* plugin)
 static void whiskermenu_free(XfcePanelPlugin*, Plugin* whiskermenu)
 {
 	delete whiskermenu;
-	whiskermenu = NULL;
 }
 
 // Wait for grab; allows modifier as shortcut
@@ -96,13 +95,15 @@ static bool panel_utils_grab_available()
 
 Plugin::Plugin(XfcePanelPlugin* plugin) :
 	m_plugin(plugin),
-	m_window(NULL)
+	m_window(NULL),
+	m_opacity(100)
 {
 	// Load settings
 	wm_settings = new Settings;
 	wm_settings->button_title = get_button_title_default();
-	wm_settings->load(g_strconcat(DATADIR, "/xfce4/whiskermenu/defaults.rc", NULL));
+	wm_settings->load(xfce_resource_lookup(XFCE_RESOURCE_CONFIG, "xfce4/whiskermenu/defaults.rc"));
 	wm_settings->load(xfce_panel_plugin_lookup_rc_file(m_plugin));
+	m_opacity = wm_settings->menu_opacity;
 
 	// Prevent empty panel button
 	if (!wm_settings->button_icon_visible)
@@ -117,12 +118,9 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 		}
 	}
 
-	// Create menu window
-	m_window = new Window;
-	g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
-
 	// Create toggle button
 	m_button = xfce_panel_create_toggle_button();
+	gtk_widget_set_name(m_button, "whiskermenu-button");
 	gtk_button_set_relief(GTK_BUTTON(m_button), GTK_RELIEF_NONE);
 	gtk_button_set_focus_on_click(GTK_BUTTON(m_button), false);
 	g_signal_connect_slot(m_button, "toggled", &Plugin::button_toggled, this);
@@ -133,7 +131,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	gtk_container_set_border_width(GTK_CONTAINER(m_button_box), 0);
 	gtk_widget_show(GTK_WIDGET(m_button_box));
 
-	m_button_icon = XFCE_PANEL_IMAGE(xfce_panel_image_new_from_source(wm_settings->button_icon_name.c_str()));
+	m_button_icon = GTK_IMAGE(gtk_image_new());
 	gtk_box_pack_start(m_button_box, GTK_WIDGET(m_button_icon), true, false, 0);
 	if (wm_settings->button_icon_visible)
 	{
@@ -174,6 +172,13 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 #else
 	orientation_changed(m_plugin, xfce_panel_plugin_get_orientation(m_plugin));
 #endif
+
+	g_signal_connect_slot<GtkWidget*,GtkStyle*>(m_button, "style-set", &Plugin::update_size, this);
+	g_signal_connect_slot<GtkWidget*,GdkScreen*>(m_button, "screen-changed", &Plugin::update_size, this);
+
+	// Create menu window
+	m_window = new Window;
+	g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
 }
 
 //-----------------------------------------------------------------------------
@@ -185,10 +190,10 @@ Plugin::~Plugin()
 	delete m_window;
 	m_window = NULL;
 
+	gtk_widget_destroy(m_button);
+
 	delete wm_settings;
 	wm_settings = NULL;
-
-	gtk_widget_destroy(m_button);
 }
 
 //-----------------------------------------------------------------------------
@@ -209,7 +214,7 @@ std::string Plugin::get_button_title() const
 
 std::string Plugin::get_button_title_default()
 {
-	return _("Applications Menu");
+	return _("Applications");
 }
 
 //-----------------------------------------------------------------------------
@@ -272,7 +277,6 @@ void Plugin::set_button_icon_name(const std::string& icon)
 {
 	wm_settings->button_icon_name = icon;
 	wm_settings->set_modified();
-	xfce_panel_image_set_from_source(m_button_icon, icon.c_str());
 	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
 }
 
@@ -302,7 +306,7 @@ void Plugin::button_toggled(GtkToggleButton* button)
 	else
 	{
 		xfce_panel_plugin_block_autohide(m_plugin, true);
-		m_window->show(m_button, xfce_panel_plugin_get_orientation(m_plugin) == GTK_ORIENTATION_HORIZONTAL);
+		show_menu(m_button, xfce_panel_plugin_get_orientation(m_plugin) == GTK_ORIENTATION_HORIZONTAL);
 	}
 }
 
@@ -328,13 +332,13 @@ void Plugin::configure()
 void Plugin::mode_changed(XfcePanelPlugin*, XfcePanelPluginMode mode)
 {
 	gtk_label_set_angle(m_button_label, (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? 270: 0);
-	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+	update_size();
 }
 #else
 void Plugin::orientation_changed(XfcePanelPlugin*, GtkOrientation orientation)
 {
 	gtk_label_set_angle(m_button_label, (orientation == GTK_ORIENTATION_VERTICAL) ? 270: 0);
-	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+	update_size();
 }
 #endif
 
@@ -353,7 +357,7 @@ gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 	}
 	else if (value && G_VALUE_HOLDS_BOOLEAN(value) && g_value_get_boolean(value))
 	{
-		m_window->show(NULL, true);
+		show_menu(NULL, true);
 	}
 	else
 	{
@@ -393,7 +397,7 @@ void Plugin::show_about()
 		"program-name", PACKAGE_NAME,
 		"translator-credits", _("translator-credits"),
 		"version", PACKAGE_VERSION,
-		"website", "http://gottcode.org/xfce4-whiskermenu-plugin/",
+		"website", PLUGIN_WEBSITE,
 		NULL);
 }
 
@@ -416,10 +420,36 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 			!wm_settings->button_title_visible,
 			0, GTK_PACK_START);
 
-	// Resize icon
+	// Load icon
 	GtkStyle* style = gtk_widget_get_style(m_button);
 	gint border = (2 * std::max(style->xthickness, style->ythickness)) + 2;
-	xfce_panel_image_set_size(m_button_icon, row_size - border);
+
+	GdkScreen* screen = gtk_widget_get_screen(GTK_WIDGET(m_plugin));
+	GtkIconTheme* icon_theme = G_LIKELY(screen != NULL) ? gtk_icon_theme_get_for_screen(screen) : NULL;
+
+#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
+	gint icon_width_max = (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL) ?
+			6 * row_size - border :
+			size - border;
+	gint icon_height_max = row_size - border;
+	GdkPixbuf* icon = xfce_panel_pixbuf_from_source_at_size(
+			wm_settings->button_icon_name.c_str(),
+			icon_theme,
+			icon_width_max,
+			icon_height_max);
+#else
+	GdkPixbuf* icon = xfce_panel_pixbuf_from_source(
+			wm_settings->button_icon_name.c_str(),
+			icon_theme,
+			row_size - border);
+#endif
+	gint icon_width = 0;
+	if (G_LIKELY(icon != NULL))
+	{
+		gtk_image_set_from_pixbuf(m_button_icon, icon);
+		icon_width = gdk_pixbuf_get_width(icon);
+		g_object_unref(G_OBJECT(icon));
+	}
 
 #if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 	if (wm_settings->button_title_visible || !wm_settings->button_single_row)
@@ -432,7 +462,7 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 		if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR &&
 				wm_settings->button_title_visible &&
 				wm_settings->button_icon_visible &&
-				label_size.width <= (size - row_size))
+				label_size.width <= (size - border - icon_width))
 		{
 			orientation = GTK_ORIENTATION_HORIZONTAL;
 		}
@@ -459,3 +489,27 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 }
 
 //-----------------------------------------------------------------------------
+
+void Plugin::update_size()
+{
+	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
+}
+
+//-----------------------------------------------------------------------------
+
+void Plugin::show_menu(GtkWidget* parent, bool horizontal)
+{
+	if (wm_settings->menu_opacity != m_opacity)
+	{
+		if ((m_opacity == 100) || (wm_settings->menu_opacity == 100))
+		{
+			delete m_window;
+			m_window = new Window;
+			g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
+		}
+		m_opacity = wm_settings->menu_opacity;
+	}
+	m_window->show(parent, horizontal);
+}
+
+//-----------------------------------------------------------------------------
